use crate::{Config, FsConfig, Global};

use std::{io, path::PathBuf, sync::Arc};

use axum::Router;
use clap::Parser;
use sea_orm_migration::MigratorTrait;
use tower_http::trace::TraceLayer;
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
pub struct Cli {
    #[arg(
        short,
        long,
        env = "RIETER_CONFIG_FILE",
        default_value = "./rieterd.toml"
    )]
    pub config_file: PathBuf,
}

impl Cli {
    pub async fn run(&self) -> crate::Result<()> {
        let config: Config = Config::figment(&self.config_file)
            .extract()
            .inspect_err(|e| tracing::error!("{}", e))?;

        tracing_subscriber::registry()
            .with(tracing_subscriber::EnvFilter::new(config.log_level.clone()))
            .with(tracing_subscriber::fmt::layer())
            .init();

        tracing::info!("Connecting to database");
        let db = crate::db::connect(&config.db).await?;

        crate::db::Migrator::up(&db, None).await?;

        let mgr = match &config.fs {
            FsConfig::Local { data_dir } => {
                crate::repo::RepoMgr::new(data_dir.join("repos"), db.clone()).await?
            }
        };

        let mgr = Arc::new(mgr);

        for _ in 0..config.pkg_workers {
            let clone = Arc::clone(&mgr);

            tokio::spawn(async move { clone.pkg_parse_task().await });
        }

        let global = Global {
            config: config.clone(),
            mgr,
            db,
        };

        // build our application with a single route
        let app = Router::new()
            .nest("/api", crate::api::router())
            .merge(crate::repo::router(&config.api_key))
            .with_state(global)
            .layer(TraceLayer::new_for_http());

        let domain: String = format!("{}:{}", config.domain, config.port)
            .parse()
            .unwrap();
        let listener = tokio::net::TcpListener::bind(domain).await?;
        // run it with hyper on localhost:3000
        Ok(axum::serve(listener, app.into_make_service())
            .await
            .map_err(|err| io::Error::new(io::ErrorKind::Other, err))?)
    }
}
